ifndef::backend-pdf[]
[#logo]
[link=https://altusmetrum.org]
image::altusmetrum-oneline.svg[Altus Metrum]
endif::[]

[license]
== License

Copyright © 2018 Bdale Garbee and Keith Packard

This document is released under the terms of the link:http://creativecommons.org/licenses/by-sa/3.0/[Creative Commons ShareAlike 3.0 License]

