/*
 * Copyright © 2008 Keith Packard <keithp@keithp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 */

void rftxrx_isr (void) __interrupt(0) __using(1)
{
}

void adc_isr (void) __interrupt(1) __using(1)
{
}

void urx0_isr (void) __interrupt(2) __using(1)
{
}

void urx1_isr (void) __interrupt(3) __using(1)
{
}

void enc_isr (void) __interrupt(4) __using(1)
{
}

void st_isr (void) __interrupt(5) __using(1)
{
}

void usb_isr (void) __interrupt(6) __using(1)
{
}

void utx0_isr (void) __interrupt(7) __using(1)
{
}

void dma_isr (void) __interrupt(8) __using(1)
{
}

void t1_isr (void) __interrupt(9) __using(1)
{
}

void t2_isr (void) __interrupt(10) __using(1)
{
}

void t3_isr (void) __interrupt(11) __using(1)
{
}

void t4_isr (void) __interrupt(12) __using(1)
{
}

void p0int_isr (void) __interrupt(13) __using(1)
{
}

void utx1_isr (void) __interrupt(14) __using(1)
{
}

void p1int_isr (void) __interrupt(15) __using(1)
{
}

void rf_isr (void) __interrupt(16) __using(1)
{
}

void wdt_isr (void) __interrupt(17) __using(1)
{
}
