= Release Notes for Version 1.8.6
include::release-head.adoc[]
:doctype: article

	Version 1.8.6

	== AltOS

	* Use maximum of 64 previous tilt values in pyro tilt tests
	
	* Eliminate 100m height requirement for coast detection

	* Change After Motor pyro check to be >= instead of ==

	* Change Time since Boost to be Time since launch.

	== AltosUI, TeleGPS

	* Clarify pyro test phrasing

	* Remove ascending/descending from pyro config UI

	* Fix accel calibration in Antenna Down mode

	* Add mag sensor reporting for TeleMega v3 idle monitoring.

	* Fix radio parameter saving at 2400 bps telmetry rate.

	== MicroPeak

	* Report altimeter-recorded maximum height value

	
