= Release Notes for Version 1.8.2
include::release-head.adoc[]
:doctype: article

	Version 1.8.2 includes support for TeleGPS version 2.0 along
	with accelerometer recalibration support in AltosUI.

	1.8.2 also contains a couple of minor fixes for AltosUI when
	analyzing saved data files.

	== AltOS

	AltOS New Features

	* Support for TeleGPS version 2.0 hardware.

	== AltosUI and TeleGPS Applications

	AltosUI and TeleGPS New Features

	* Support for TeleGPS version 2.0.

	* Accelerometer re-calibration user interface.

	AltosUI and TeleGPS Bug Fixes

	* Prevent some crashes when reading older saved flight data
          for graphing or KML export.
