= Release Notes for Version 1.8.5
include::release-head.adoc[]
:doctype: article

	Version 1.8.5 includes fixes to the ground software support
	for TeleBT v4, along with a few other minor updates.

	== AltOS

	* Fix startup beeps that indicate sensor failures.

	== AltosUI, TeleGPS

	* When updating device firmware, make sure selected firmware
          matches target device.

	* Correct Bluetooth device matching when looking for TeleBT
          devices.
