= Release Notes for Version 1.4.2
include::release-head.adoc[]
:doctype: article

	Version 1.4.2 is a minor release. It fixes Java-related install issues on
	Windows

	== AltosUI and TeleGPS Applications

	Windows Install Fixes

	* Checks for Java installation data in more registry locations.

	* Allows user to bypass Java installation in case the
	  detection fails.
