/* Copyright (C) 2000-2009 Peter Selinger.
   This file is part of ccrypt. It is free software and it is covered
   by the GNU general public license. See the file COPYING for details. */

extern const word8x4 M0[4][256];
extern const word8x4 M1[4][256];
extern const int xrcon[30];
extern const word8 xS[256];
extern const word8 xSi[256];

