= Release Notes for Version 1.9.2
include::release-head.adoc[]
:doctype: article

	Version 1.9.2

	== AltOS

	* Add support for TeleMega v4.0

	* Fix time wrapping issue with TeleLCO and TeleFire

	== AltosUI, TeleGPS, MicroPeak

	* Add column in AltosUI for IMU data from TeleMega and EasyMega

	== AltosDroid

	* Allow sorting of trackers by call, serial, frequency or age

	* Offer selection of font sizes

	* Various bug fixes for older Android versions
