= Release Notes for Version 1.9
include::release-head.adoc[]
:doctype: article

	Version 1.9

	== AltOS

	* Add support for EasyMega v2.0

	* Replace C library for ARM devices

	* Remove support for 8051-based devices and SDCC compiler
	
	== AltosUI, TeleGPS, MicroPeak

	* Select ROM images based on product name when reflashing

	* Fix TeleGPS v2 battery voltage display
	
