[appendix]
== Release Notes
	:leveloffset: 2
	include::release-notes-1.9.6.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.9.5.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.9.4.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.9.3.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.9.2.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.9.1.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.9.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.8.7.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.8.6.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.8.5.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.8.4.adoc[]

	<<<<

	:leveloffset: 2
	include::release-notes-1.8.3.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.8.2.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.8.1.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.8.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.7.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.6.8.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.6.5.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.6.4.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.6.3.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.6.2.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.6.1.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.6.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.5.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.4.2.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.4.1.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.4.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.3.2.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.3.1.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.3.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.2.1.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.2.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.1.1.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.1.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-1.0.1.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-0.9.2.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-0.9.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-0.8.adoc[]

	<<<<
	:leveloffset: 2
	include::release-notes-0.7.1.adoc[]

	:leveloffset: 0
