= Release Notes for Version 0.8
include::release-head.adoc[]
:doctype: article

	Version 0.8 offers a major upgrade in the AltosUI
	interface.

	== AltosUI Application:  

    	* Post-flight graphing tool. This lets you explore the
	  behaviour of your rocket after flight with a scroll-able and
	  zoom-able chart showing the altitude, speed and acceleration
	  of the airframe along with events recorded by the flight
	  computer. You can export graphs to PNG files, or print them
	  directly.

    	* Real-time moving map which overlays the in-progress flight
	  on satellite imagery fetched from Google Maps. This lets you
	  see in pictures where your rocket has landed, allowing you
	  to plan recovery activities more accurately.
    
    	* Wireless recovery system testing. Prep your rocket for
	  flight and test fire the deployment charges to make sure
	  things work as expected. All without threading wires through
	  holes in your airframe.
    
    	* Optimized flight status displays. Each flight state now has
	  it's own custom 'tab' in the flight monitoring window so you
	  can focus on the most important details. Pre-flight, the
	  system shows a set of red/green status indicators for
	  battery voltage, apogee/main igniter continutity and GPS
	  reception. Wait until they're all green and your rocket is
	  ready for flight. There are also tabs for ascent, descent
	  and landing along with the original tabular view of the
	  data.
    
    	* Monitor multiple flights simultaneously. If you have more
	  than one TeleDongle, you can monitor a flight with each one
	  on the same computer.
    
    	* Automatic flight monitoring at startup. Plug TeleDongle into
	  the machine before starting AltosUI and it will
	  automatically connect to it and prepare to monitor a flight.
    
    	* Exports Google Earth flight tracks. Using the Keyhole Markup
	  Language (.kml) file format, this provides a 3D view of your
	  rocket flight through the Google Earth program.
