[dedication]
== Acknowledgments

	Thanks to Anthony (AJ) Towns for major contributions including
	the TeleGPS graphing and site map code and associated documentation.
	Free software means that our customers and friends can become our
	collaborators, and we certainly appreciate this level of
	contribution!

	Have fun using these products, and we hope to meet all of you
	out on the rocket flight line somewhere.

	[verse]
	Bdale Garbee, KB0G
	NAR #87103, TRA #12201

	[verse]
	Keith Packard, KD7SQG
	NAR #88757, TRA #12200
